// ============================================================================
// BDreaderGAMEBDmain.cpp
// ----------------------------------------------------------------------------
// (c) 2022-2024 by HerzAusGold
//               Hubert Haberkorn
//               herz-aus-gold-42@gmx.de
//               http://www.arthurdent.de/
// ----------------------------------------------------------------------------
// The above copyright notice and this permission notice 
// shall be included in all copies or substantial portions 
// of the Software.
// ----------------------------------------------------------------------------
// read file dump from "BoulderDsh" (by Guido Mersmann) 
// and extract level data to -> ".gamebd" file
// generate "levelinfo.conf" with some extra info
// ============================================================================

#include "stdio.h"

#include <string>
#include <iostream>
#include <vector>
#include <filesystem>

#include "BDreaderGAMEBDmain.h"
#include "BDreaderGAMEBDutil.h"
#include "BDreaderGAMEBDreadLevelGAMEBD.h"
#include "BDreaderGAMEBDreadLevelMagic64.h"

namespace filesys = std::filesystem;

using namespace std;


/*
 * Get the list of all files in given directory and its sub directories.
 *
 * Arguments
 *     dirPath : Path of directory to be traversed
 *     dirSkipList : List of folder names to be skipped
 *
 * Returns:
 *     vector containing paths of all the files in given directory and its sub directories
 *
 */
std::vector<std::string> getAllFilesInDir(const std::string &dirPath, const std::string ext, const std::vector<std::string> dirSkipList = { })
{
    // Create a vector of string
    std::vector<std::string> listOfFiles;
    try {
        // Check if given path exists and points to a directory
        if (filesys::exists(dirPath) && filesys::is_directory(dirPath))
        {
            // Create a Recursive Directory Iterator object and points to the starting of directory
            filesys::recursive_directory_iterator iter(dirPath);
            // Create a Recursive Directory Iterator object pointing to end.
            filesys::recursive_directory_iterator end;
            // Iterate till end
            while (iter != end)
            {
                // Check if current entry is a directory and if exists in skip list
                if (filesys::is_directory(iter->path()) &&
                        (std::find(dirSkipList.begin(), dirSkipList.end(), iter->path().filename()) != dirSkipList.end()))
                {
                    // Skip the iteration of current directory pointed by iterator
#ifdef USING_BOOST
                    // Boost Fileystsem  API to skip current directory iteration
                    iter.no_push();
#else
                    // c++17 Filesystem API to skip current directory iteration
                    iter.disable_recursion_pending();
#endif
                }
                else
                {
                    string tmpname = iter->path().string();
                    string tmpext  = iter->path().extension().string();
                    if (tmpext == ext) {
                        // Add the name in vector
                        listOfFiles.push_back(iter->path().string());
                    }
                }
                error_code ec;
                // Increment the iterator to point to next entry in recursive iteration
                iter.increment(ec);
                if (ec) {
                    std::cerr << "Error While Accessing : " << iter->path().string() << " :: " << ec.message() << '\n';
                }
            }
        }
    }
    catch (std::system_error & e)
    {
        std::cerr << "Exception :: " << e.what();
    }
    return listOfFiles;
}



/*---- readLevel -----------------------------------------------------------------*/
int main(int argc, char *argv[]) 
{
#ifndef VIEWER_ONLY
    byte_t        bytes[4]; 

    //--- Testarea ---
    cout << "--- Testarea ---" << endl;
    char hexstr[] = "0128";
    cout << "dec: " << hex2dec(hexstr) << endl;
    int deci = hex2dec(hexstr);
    cout << "<< hex   : " << std::hex << deci << std::dec << endl;
    cout << "dec2hex  : " << dec2hex(deci) << endl;
    cout << "tohexstr4: " << tohexstr(deci,4) << " -> " << hex2dec(tohexstr(deci,4))<< endl;
    cout << "tohexstr2: " << tohexstr(deci,2) << " -> " << hex2dec(tohexstr(deci,2))<< endl;
    HexToBytes(&bytes[0], tohexstr(deci,4).c_str(), 4);
    outbytes("HexToBytes4", bytes, 4);
    HexToBytes(&bytes[0], tohexstr(deci,2).c_str(), 4);
    outbytes("HexToBytes2", bytes, 2);
    DecToBytes(&bytes[0], deci, 4);
    outbytes("DecToBytes4", bytes, 4);
    DecToBytes(&bytes[0], deci, 2);
    outbytes("DecToBytes2", bytes, 2);
#endif

    //--- Filename ---
    string filename  = "";
    string offsetStr = "10";
    int    doWrite   = 0;
    int    levelNo   = -1;

    if (argc > 2) {
        levelNo = atoi(argv[2]);
    } else {
        levelNo = -1;
    }
    if (argc > 1) {
        filename = string(argv[1]);
    } else {
        char cfilename[1000] = {0};
        cerr << "Filename          : ";
        cin.getline(cfilename, sizeof(cfilename), '\n');
        filename = string(cfilename);
        cerr << "LevelNo (-1 = all): ";
        cin >> levelNo;

        //filename = "S:\\_transfer_\\SomeStuff\\Games\\NoOne\\NoOne1_Level1.game";
//#ifndef VIEWER_ONLY
        if (filename == "M") {
            filename = "C:\\Users\\Public\\Documents\\Amiga Files\\Shared\\AmigaHD\\Magic64\\MagiC64\\P00FILES\\";
            filename = filename + "boulderdash1-A-25--2.raw";  // 2
        }
        if (filename == "B") {
            filename = "S:\\_transfer_\\SomeStuff\\Games\\BoulderDash\\";
            filename = filename + "BDash_I_Level1.game";
        }
        if (filename == "-") {
            filename = "S:\\_transfer_\\SomeStuff\\Games\\ThomasRichter\\TRI_BD.game";
            // filename = "S:\\_transfer_\\SomeStuff\\Games\\MarkusSauermann\\MBD.game";
        }
        if (filename == "+") {
            filename = "S:\\_transfer_\\SomeStuff\\GameBD\\ThomasRichter\\TRI_BD\\TRI_BD.gamebd";
            // filename = "S:\\_transfer_\\SomeStuff\\Games\\MarkusSauermann\\MBD.gamebd";
        }

        cerr << "Offset: ";
        cin >> offsetStr;
        if (offsetStr == "-") {
            offsetStr = "000B48A2";
            // offsetStr = "000B491A";
        }
        if (offsetStr == "+") {
            offsetStr = "10";
        }
        cerr << "DoWrite: ";
        cin >> doWrite;
//#endif
    } 

    // special quirk
    if(strstr(filename.c_str(), ".gamebd") == NULL) {
        if(strstr(filename.c_str(), ".game") != NULL) 
        if(strstr(filename.c_str(), "\\SomeStuff\\") != NULL) {
            // offsetStr = "000B48A2";
            offsetStr = "000B4800";
        }
    }

    cout << endl;
    cout << "-- Filename: " << filename << endl;
    cout << "-- LevelNo : " << levelNo  << endl;

    int games  = 0;
    int levels = 0;
//#ifndef VIEWER_ONLY
    if (filename == "*") {
        offsetStr = "000b4800";
        filename  = "S:\\_transfer_\\SomeStuff\\Games\\Augs\\";
        filename  = "S:\\_transfer_\\SomeStuff\\Games\\";
        std::vector<std::string> files = getAllFilesInDir(filename, ".game");
        for (int i=0; i < files.size(); i++) {
            levels += doReadLevelGAMEBD(files[i], offsetStr, levelNo, doWrite);
        }
        games = files.size();
    } else if (filename == "#") {
        offsetStr = "10";
        filename  = "S:\\_transfer_\\SomeStuff\\GAMEBD\\Augs\\Augs\\";
        filename  = "S:\\_transfer_\\SomeStuff\\GAMEBD\\";
        std::vector<std::string> files = getAllFilesInDir(filename, ".gamebd");
        for (int i=0; i < files.size(); i++) {
            levels += doReadLevelGAMEBD(files[i], offsetStr, levelNo, doWrite);
        }
        games = files.size();
    } else {
//#endif
      if (strstr(filename.c_str(), ".game")) {
          levels += doReadLevelGAMEBD(filename, offsetStr, levelNo, doWrite);
          games  = 1;
      } else {
        levels += readLevelMagic64(filename, offsetStr, levelNo, doWrite);
        games  = 1;
      }
//#ifndef VIEWER_ONLY
    }
//#endif

    cout << string(80, '=') << endl;
    cout << "== Games : " << games  << endl;
    cout << "== Levels: " << levels << endl;
    cout << string(80, '=') << endl;

    cerr << "-- done --" << endl;
    if (levelNo <= -1) {
        cerr << "-- <Press Key> --" << endl;
        getchar();
        cerr << "-- <Press Key> --" << endl;
        getchar();
    }
    return 0;
}
